class KnowledgeController < ApplicationController
  # ApplicationController < ActionController::Base (in application.rb)
  layout "gfdnavi"
  before_filter :login_required, :except => [:index, :list, :show]

  def index
    list
    render :action => 'list'
  end

  # GETs should be safe (see http://www.w3.org/2001/tag/doc/whenToUseGet.html)
  verify :method => :post, :only => [ :destroy, :create, :update ], :redirect_to => { :action => :list }

  def list
    @knowledge_pages, @knowledges = paginate :knowledges, :per_page => 30
    @knowledges = Knowledge.find_for_user(session[:user])
  end

  def show
    @knowledge = Knowledge.find_for_user(session[:user], params[:id])
    @knowledge_figures = @knowledge.knowledge_figures
  end

  def new
    @knowledge = Knowledge.new
    @groups = Group.find_for_user(session[:user])
    @user_name = session[:user].login
  end

  def create
    ## < knowledges table >
    if params[:commit]=="Save" # insert all original data include "@Knowledge.id"
      unless @knowledge = Knowledge.find_for_owner(session[:user], params[:id])
        raise "Specified knowledge is not found or not yours."
      end
    else
      @knowledge = Knowledge.new
    end

    # insert title, textbody, user_id
    @knowledge.attributes = {"title" => params[:knowledge]["title"]}
    @knowledge.attributes = {"textbody" => params[:knowledge]["textbody"]}
    @knowledge.attributes = {"user_id" => session[:user].id}

    # insert group_id
    if params[:knowledge]["group_name"] == "everyone"
      @knowledge.attributes = {"group_id" => nil}
    else
      unless group = Group.find(:first, :conditions => ["name = ? and user_id = ?", params[:knowledge]["group_name"], session[:user].id])
        raise "Specified group is not found."
      end
      @knowledge.attributes = {"group_id" => group.id}
    end

    # if "Save", insert original path
    if params[:commit]=="Save"
      @knowledge.attributes = {"knowledge_path" => params[:origin_knowledge_path]}
    else # if "Save As", "Create", insert knowledge_path (relative path -> full path)
      if params[:knowledge]["knowledge_path"] != ""
        fullpath = "/usr/" + session[:user].login + "/knowledge/" + params[:knowledge]["knowledge_path"] + ".yml"
        @knowledge.attributes = {"knowledge_path" => fullpath}
      else
        raise "knowledge_path is empty!"
      end
    end
    
    # if "Save As", "Create", save (and insert id automatically)
    unless params[:commit]=="Save"
      unless @knowledge.save
        raise "Error occured in knowledge_path. Maybe you use pathphrase that used before."
      end
    end
    
    ## < knowledge_figures table >
    # if "Save", at first keep original knowledge_figures. (delete them later.)
    if params[:commit]=="Save"
      old_knowledge_figures = KnowledgeFigure.find(:all, :conditions => ["knowledge_id = ?", params[:id]])
    end

    # extract sorted [caption0, caption1, ...] and sorted [figure_path0, figure_path1, ...] from Hash
    already_saved_figure_ids = []
    hash_cap_and_figpath = params[:knowledge_figure]
    if hash_cap_and_figpath
      captions = hash_cap_and_figpath.keys
      captions.delete_if{|x| x !~ /^caption/}
      sort_captions = captions.sort.collect{|k| hash_cap_and_figpath[k]}
      figure_paths = hash_cap_and_figpath.keys
      figure_paths.delete_if{|x| x !~ /^figure_path/}
      sort_figpaths = figure_paths.sort.collect{|k| hash_cap_and_figpath[k]}

      # check length of [caption0, caption1, ...] and [figure_path0, figure_path1, ...]
      if sort_captions.length != sort_figpaths.length
        raise "Error occured in caption or figure_path."
      end
      
      # save caption and figure_path in knowledge_figures table
      nfigs = sort_captions.length - 1
      unless ((nfigs == 0) && (sort_captions[0] == "") && (sort_figpaths[0] == ""))
        while ((sort_captions[nfigs] == "") && (sort_figpaths[nfigs] == ""))
          nfigs = nfigs - 1
        end
        
        for count in 0..nfigs
          print "count[", count, "]\n"
          if (sort_figpaths[count] == "")
            delete_knowledge_figures
            raise "Error. Figure_path are nil."
          end
          figpath = sort_figpaths[count]
          unless fig = Variable.find_for_user(session[:user], :first, :conditions => ["path = ?", figpath])
            delete_knowledge_figures
            raise "Figure #{figpath} not found"
          end
          if fig.gphysizable?
            delete_knowledge_figures
            raise "#{figpath} is a GPhys, not a figure"
          end
          kfpar = {"knowledge_id" => @knowledge.id, "caption" => sort_captions[count], "variable_id" => fig.id}
          @knowledge_fig = KnowledgeFigure.new(kfpar)
          
          if @knowledge_fig.save
            flash[:notice] = 'Knowledge was successfully created.'
            already_saved_figure_ids.insert(-1, @knowledge_fig.id)
          else
            delete_knowledge_figures
            redirect_to :action => 'new'
          end
        end
      end
    end

    if params[:commit]=="Save"
      # if "Save", save.
      unless @knowledge.save
        raise "Error occured in knowledge_path. Maybe you use pathphrase that used before."
      end
      # if "Save", delete original knowledge_figures.
      for old_kf in old_knowledge_figures
        old_kf.destroy
      end
    end

    redirect_to :action => 'list'
  end

def delete_knowledge_figures
  if params[:commit]=="Save"
    for ids in already_saved_figure_ids
      figure = KnowledgeFigure.find(ids)
      figure.destroy
    end
  else
    @knowledge.destroy
  end
end
  
  def edit
    @knowledge = Knowledge.find_for_owner(session[:user], params[:id])
    @knowledge_figures = @knowledge.knowledge_figures
    @groups = Group.find_for_user(session[:user])
    @captions = []
    @figure_paths = []
    @knowledge_figures.each do |kf|
      if kf.knowledge_id.to_s == params[:id].to_s
        @captions << kf.caption
        @figure_paths << Variable.find_for_user(session[:user], kf.variable_id).path
      end
    end
    if @captions.length == @figure_paths.length
        @num_of_figure = @captions.length
    else
      raise "Unknown error."
    end
  end

  def destroy
    @knowledge = Knowledge.find_for_owner(session[:user], params[:id])
    @knowledge.destroy
    redirect_to :action => 'list'
  end
end
