require File.dirname(__FILE__) + '/../test_helper'
require File.expand_path(File.dirname(__FILE__) + "/../../lib/narray_yaml")

class VariableTest < Test::Unit::TestCase
  fixtures :variables, :directories, :keyword_attributes, :spatial_attributes,
           :users, :groups, :group_members

  def setup
    @var = Variable.find(2)
  end

  def test_create
    u = variables(:ncep_jan_u)
    assert_kind_of Variable, @var
    assert_equal u.id, @var.id
    assert_equal u.directory_id, @var.directory_id
    assert_equal u.path, @var.path
    assert_equal u.starttime, @var.starttime
    assert_equal u.endtime, @var.endtime
    assert_equal u.name, @var.name
  end

  def test_directory
    assert_kind_of Directory, @var.directory
    assert_equal directories(:ncep_uv_jan), @var.directory
  end

  def test_keyword_attribute
    assert_kind_of Array, @var.keyword_attributes
    assert_kind_of KeywordAttribute, @var.keyword_attributes[0]
  end

  def test_spatial_attribute
    assert_kind_of Array, @var.spatial_attributes
    assert_kind_of SpatialAttribute, @var.spatial_attributes[0]
    size = @var.keyword_attributes.size
    name = "test"
    value = "this is test attribute"
    @var.keyword_attributes.create(:name => name, :value => value)
    assert_equal size+1, @var.keyword_attributes.size
    ka = @var.keyword_attributes.find(:first, :conditions => ['name = ? and value = ?', name, value])
    assert_kind_of KeywordAttribute, ka
    ka = KeywordAttribute.find(:first, :conditions => ['name = ? and value = ?', name, value])
    assert_kind_of KeywordAttribute, ka
    @var.keyword_attributes.delete(ka)
    assert_equal nil, KeywordAttribute.find(:first, :conditions => ['name = ? and value = ?', name, value])
  end

  def test_find_for_user
    vars = Variable.find_for_user(nil, :all)
    varnames =  vars.collect{|x| x.name}
    assert_equal varnames, ["T"]

    user = User.find_by_login('bob')

    vars = Variable.find_for_user(user, :all, {:order =>"name DESC"})
    varnames =  vars.collect{|x| x.name}
    assert_equal varnames, ["V", "U", "T"]

    vars = Variable.find_for_user(user, :all, {:conditions =>"name = 'V'"})
    varnames =  vars.collect{|x| x.name}
    assert_equal varnames, ["V"]

    vars = Variable.find_for_user(user, 1, 3)  #find: id syntax: id,..; [id,..]
    varnames =  vars.collect{|x| x.name}
    assert_equal varnames, ["T", "V"]

  end

end
